@@ -1,4 +1,4 @@
-/* Copyright 2009-2015,2018-2019,2023
+/* Copyright 2009-2015,2018-2019
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -22,11 +22,12 @@
 # include <config.h>
 #endif
 
-#include <limits.h>
+#ifdef HAVE_LIMITS_H
+# include <limits.h>
+#endif
+
 #include <byteswap.h>
-#include <intprops.h>
 #include <errno.h>
-#include <stdio.h>
 #include <striconveh.h>
 #include <uniconv.h>
 #include <unistr.h>
@@ -53,7 +54,6 @@
 #include "numbers.h"
 #include "pairs.h"
 #include "ports.h"
-#include "private-options.h"
 #include "srfi-4.h"
 #include "strings.h"
 #include "symbols.h"
@@ -328,69 +328,6 @@ scm_c_take_typed_bytevector (signed char *contents, size_t len,
   return ret;
 }
 
-SCM_DEFINE (scm_bytevector_slice, "bytevector-slice", 2, 1, 0,
-            (SCM bv, SCM offset, SCM size),
-            "Return the slice of @var{bv} starting at @var{offset} and counting\n"
-            "@var{size} bytes.  When @var{size} is omitted, the slice covers all\n"
-            "of @var{bv} starting from @var{offset}.  The returned slice shares\n"
-            "storage with @var{bv}: changes to the slice are visible in @var{bv}\n"
-            "and vice-versa.\n"
-            "\n"
-            "When @var{bv} is actually a SRFI-4 uniform vector, its element\n"
-            "type is preserved unless @var{offset} and @var{size} are not aligned\n"
-            "on its element type size.\n")
-#define FUNC_NAME s_scm_bytevector_slice
-{
-  SCM ret;
-  size_t c_offset, c_size;
-  scm_t_array_element_type element_type;
-
-  SCM_VALIDATE_BYTEVECTOR (1, bv);
-
-  c_offset = scm_to_size_t (offset);
-
-  if (SCM_UNBNDP (size))
-    {
-      if (c_offset < SCM_BYTEVECTOR_LENGTH (bv))
-        c_size = SCM_BYTEVECTOR_LENGTH (bv) - c_offset;
-      else
-        c_size = 0;
-    }
-  else
-    c_size = scm_to_size_t (size);
-
-  if (c_offset == 0 && c_size == SCM_BYTEVECTOR_LENGTH (bv))
-    return bv;
-
-  if (INT_ADD_OVERFLOW (c_offset, c_size)
-      || (c_offset + c_size > SCM_BYTEVECTOR_LENGTH (bv)))
-    scm_out_of_range (FUNC_NAME, offset);
-
-  /* Preserve the element type of BV, unless we're not slicing on type
-     boundaries.  */
-  element_type = SCM_BYTEVECTOR_ELEMENT_TYPE (bv);
-  if ((c_offset % SCM_BYTEVECTOR_TYPE_SIZE (bv) != 0)
-      || (c_size % SCM_BYTEVECTOR_TYPE_SIZE (bv) != 0))
-    element_type = SCM_ARRAY_ELEMENT_TYPE_VU8;
-  else
-    c_size /= (scm_i_array_element_type_sizes[element_type] / 8);
-
-  ret = make_bytevector_from_buffer (c_size,
-                                     SCM_BYTEVECTOR_CONTENTS (bv) + c_offset,
-                                     element_type);
-  if (!SCM_MUTABLE_BYTEVECTOR_P (bv))
-    {
-      /* Preserve the immutability property.  */
-      scm_t_bits flags = SCM_BYTEVECTOR_FLAGS (ret);
-      SCM_SET_BYTEVECTOR_FLAGS (ret, flags | SCM_F_BYTEVECTOR_IMMUTABLE);
-    }
-
-  SCM_BYTEVECTOR_SET_PARENT (ret, bv);
-
-  return ret;
-}
-#undef FUNC_NAME
-
 /* Shrink BV to C_NEW_LEN (which is assumed to be smaller than its current
    size) and return the new bytevector (possibly different from BV).  */
 SCM
@@ -490,8 +427,8 @@ scm_c_bytevector_set_x (SCM bv, size_t index, uint8_t value)
 
 
 
-static int
-print_bytevector (SCM bv, SCM port, scm_print_state *pstate SCM_UNUSED)
+int
+scm_i_print_bytevector (SCM bv, SCM port, scm_print_state *pstate SCM_UNUSED)
 {
   ssize_t ubnd, inc, i;
   scm_t_array_handle h;
@@ -513,86 +450,6 @@ print_bytevector (SCM bv, SCM port, scm_print_state *pstate SCM_UNUSED)
   return 1;
 }
 
-
-
-static int
-print_bytestring (SCM bv, SCM port, scm_print_state *pstate SCM_UNUSED)
-{
-  assert (scm_is_bytevector (bv));
-
-  const size_t c_len = SCM_BYTEVECTOR_LENGTH (bv);
-  const uint8_t * const c_bv = (uint8_t *) SCM_BYTEVECTOR_CONTENTS (bv);
-
-  scm_puts ("#u8\"", port);
-  for (size_t i = 0; i < c_len; i++)
-    {
-      switch (c_bv[i])
-        {
-        case '\n': scm_puts("\\n", port); break;
-        case '\r': scm_puts("\\r", port); break;
-        case '"': scm_puts("\\\"", port); break;
-        case '\\': scm_puts("\\\\", port); break;
-        case '\t': scm_puts("\\t", port); break;
-        case '|': scm_puts("\\|", port); break;
-        case '\a': scm_puts("\\a", port); break;
-        case '\b': scm_puts("\\b", port); break;
-        case '\0': scm_puts("\\x0;", port); break;
-        case 0x01: scm_puts("\\x1;", port); break;
-        case 0x02: scm_puts("\\x2;", port); break;
-        case 0x03: scm_puts("\\x3;", port); break;
-        case 0x04: scm_puts("\\x4;", port); break;
-        case 0x05: scm_puts("\\x5;", port); break;
-        case 0x06: scm_puts("\\x6;", port); break;
-        case 0x0b: scm_puts("\\xb;", port); break;
-        case 0x0c: scm_puts("\\xc;", port); break;
-        case 0x0e: scm_puts("\\xe;", port); break;
-        case 0x0f: scm_puts("\\xf;", port); break;
-        case 0x10: scm_puts("\\x10;", port); break;
-        case 0x11: scm_puts("\\x11;", port); break;
-        case 0x12: scm_puts("\\x12;", port); break;
-        case 0x13: scm_puts("\\x13;", port); break;
-        case 0x14: scm_puts("\\x14;", port); break;
-        case 0x15: scm_puts("\\x15;", port); break;
-        case 0x16: scm_puts("\\x16;", port); break;
-        case 0x17: scm_puts("\\x17;", port); break;
-        case 0x18: scm_puts("\\x18;", port); break;
-        case 0x19: scm_puts("\\x19;", port); break;
-        case 0x1a: scm_puts("\\x1a;", port); break;
-        case 0x1b: scm_puts("\\x1b;", port); break;
-        case 0x1c: scm_puts("\\x1c;", port); break;
-        case 0x1d: scm_puts("\\x1d;", port); break;
-        case 0x1e: scm_puts("\\x1e;", port); break;
-        case 0x1f: scm_puts("\\x1f;", port); break;
-        default:
-          if (c_bv[i] <= 0x7e)
-            scm_putc (c_bv[i], port);
-          else
-            {
-              char digits[3];
-              const int n = snprintf(digits, 3, "%2x", c_bv[i]);
-              assert (n == 2);
-              scm_puts ("\\x", port);
-              scm_puts (digits, port);
-              scm_puts (";", port);
-            }
-        }
-    }
-  scm_putc ('"', port);
-  return 1;
-}
-
-
-
-int
-scm_i_print_bytevector (SCM bv, SCM port, scm_print_state *pstate SCM_UNUSED)
-{
-  if (!SCM_PRINT_BYTESTRINGS_P)
-    return print_bytevector(bv, port, pstate);
-  else
-    return print_bytestring(bv, port, pstate);
-}
-
-
 
 /* General operations.  */
 
@@ -708,57 +565,39 @@ SCM_DEFINE (scm_bytevector_eq_p, "bytevector=?", 2, 0, 0,
 }
 #undef FUNC_NAME
 
-static SCM scm_bytevector_fill_partial_x (SCM bv, SCM fill, SCM start, SCM end);
-
-SCM_DEFINE (scm_bytevector_fill_partial_x, "bytevector-fill!", 2, 2, 0,
-	    (SCM bv, SCM fill, SCM start, SCM end),
-	    "Fill positions [@var{start} ... @var{end}) of bytevector "
-            "@var{bv} with @var{fill}, a byte. @var{start} defaults to 0 "
-            "and @var{end} defaults to the length of @var{bv}. "
-            "The return value is unspecified.")
-#define FUNC_NAME s_scm_bytevector_fill_partial_x
+SCM_DEFINE (scm_bytevector_fill_x, "bytevector-fill!", 2, 0, 0,
+	    (SCM bv, SCM fill),
+	    "Fill bytevector @var{bv} with @var{fill}, a byte.")
+#define FUNC_NAME s_scm_bytevector_fill_x
 {
+  size_t c_len, i;
+  uint8_t *c_bv, c_fill;
+  int value;
+
   SCM_VALIDATE_MUTABLE_BYTEVECTOR (1, bv);
 
-  int value  = scm_to_int (fill);
+  value = scm_to_int (fill);
   if (SCM_UNLIKELY ((value < -128) || (value > 255)))
     scm_out_of_range (FUNC_NAME, fill);
-  
-  size_t i = 0;
-  size_t c_end = SCM_BYTEVECTOR_LENGTH (bv);
-  uint8_t *c_bv = (uint8_t *) SCM_BYTEVECTOR_CONTENTS (bv);
+  c_fill = (uint8_t) value;
 
-  if (!SCM_UNBNDP (start))
-    i = scm_to_unsigned_integer (start, 0, c_end);
-  if (!SCM_UNBNDP (end))
-    c_end = scm_to_unsigned_integer (end, i, c_end);
+  c_len = SCM_BYTEVECTOR_LENGTH (bv);
+  c_bv = (uint8_t *) SCM_BYTEVECTOR_CONTENTS (bv);
 
-  memset (c_bv + i, value, c_end-i);
+  for (i = 0; i < c_len; i++)
+    c_bv[i] = c_fill;
 
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-SCM
-scm_bytevector_fill_x (SCM bv, SCM fill)
-#define FUNC_NAME s_scm_bytevector_fill_x
-{
-  return scm_bytevector_fill_partial_x (bv, fill, SCM_UNDEFINED, SCM_UNDEFINED);
-}
-#undef FUNC_NAME
-
-
 SCM_DEFINE (scm_bytevector_copy_x, "bytevector-copy!", 5, 0, 0,
 	    (SCM source, SCM source_start, SCM target, SCM target_start,
 	     SCM len),
 	    "Copy @var{len} bytes from @var{source} into @var{target}, "
-	    "reading from a block starting at @var{source_start} (a positive "
-            "index within @var{source}) and writing to a block starting at "
-	    "@var{target_start}.\n\n"
-            "It is permitted for the @var{source} and @var{target} regions to "
-            "overlap. In that case, copying takes place as if the source is "
-            "first copied into a temporary bytevector and then into the "
-            "destination. ")
+	    "starting reading from @var{source_start} (a positive index "
+	    "within @var{source}) and start writing at "
+	    "@var{target_start}.")
 #define FUNC_NAME s_scm_bytevector_copy_x
 {
   size_t c_len, c_source_len, c_target_len;
