@@ -1,4 +1,4 @@
-/* Copyright 1996-2002,2004,2006,2009-2014,2016-2018
+/* Copyright 1996-2002,2004,2006,2009-2019,2021
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -1454,7 +1454,90 @@ SCM_DEFINE (scm_umask, "umask", 0, 1, 0,
 #undef FUNC_NAME
 
 SCM_INTERNAL SCM scm_i_mkstemp (SCM, SCM);
-SCM_DEFINE (scm_i_mkstemp, "mkstemp!", 1, 1, 0,
+SCM_DEFINE (scm_i_mkstemp, "mkstemp", 1, 1, 0,
+	    (SCM tmpl, SCM mode),
+	    "Create a new unique file in the file system.  Return\n"
+	    "a buffered port open for reading and writing to the file.\n"
+	    "\n"
+	    "@var{tmpl} is a string specifying where the file should be\n"
+	    "created: it must end with @samp{XXXXXX}.  The name of the\n"
+            "newly created file will be the same as @var{tmpl}, but with\n"
+            "those @samp{X}s changed, and can be determined by calling\n"
+            "@code{port-filename} on the returned port.\n"
+            "\n"
+            "Note that the newly created file is not deleted automatically\n"
+            "by Guile; probably the caller should arrange to call\n"
+            "@code{delete-file} when the file is no longer needed.\n"
+            "\n"
+	    "POSIX doesn't specify the permissions mode of the file.\n"
+            "On GNU and most systems it's @code{#o600}.  An application can\n"
+            "use @code{chmod} to relax that if desired.  For example\n"
+	    "@code{#o666} less @code{umask}, which is usual for ordinary\n"
+	    "file creation,\n"
+	    "\n"
+	    "@example\n"
+	    "(let ((port (mkstemp \"/tmp/myfile-XXXXXX\")))\n"
+	    "  (chmod port (logand #o666 (lognot (umask))))\n"
+	    "  ...)\n"
+	    "@end example\n"
+            "\n"
+            "The optional @var{mode} argument specifies a mode, as a string\n"
+            "in the same format that @code{open-file} takes.  It defaults\n"
+            "to @code{\"w+\"}.")
+#define FUNC_NAME s_scm_i_mkstemp
+{
+  char *c_tmpl;
+  long mode_bits;
+  int rv;
+  int open_flags, is_binary;
+  SCM port;
+
+  SCM_VALIDATE_STRING (SCM_ARG1, tmpl);
+  if (!SCM_UNBNDP (mode))
+    SCM_VALIDATE_STRING (SCM_ARG2, mode);
+
+  scm_dynwind_begin (0);
+
+  c_tmpl = scm_to_locale_string (tmpl);
+  scm_dynwind_free (c_tmpl);
+  if (SCM_UNBNDP (mode))
+    {
+      /* mkostemp will create a read/write file and add on additional
+         flags; open_flags just adjoins flags to that set.  */
+      open_flags = 0;
+      is_binary = 0;
+      mode_bits = SCM_OPN | SCM_RDNG | SCM_WRTNG;
+    }
+  else
+    {
+      open_flags = scm_i_mode_to_open_flags (mode, &is_binary, FUNC_NAME);
+      /* mkostemp(2) only defines O_APPEND, O_SYNC, and O_CLOEXEC to be
+         useful, as O_RDWR|O_CREAT|O_EXCL are implicitly added.  It also
+         notes that other flags may error on some systems, which turns
+         out to be the case.  Of those flags, O_APPEND and O_BINARY are
+         the only ones of interest anyway, so limit to those flags.  */
+      open_flags &= O_APPEND | O_BINARY;
+      mode_bits = scm_i_mode_bits (mode);
+    }
+
+  SCM_SYSCALL (rv = mkostemp (c_tmpl, open_flags));
+  if (rv == -1)
+    SCM_SYSERROR;
+
+  SCM name = scm_from_locale_string (c_tmpl);
+  scm_dynwind_end ();
+
+  port = scm_i_fdes_to_port (rv, mode_bits, name, 0);
+  if (is_binary)
+    /* Use the binary-friendly ISO-8859-1 encoding. */
+    scm_i_set_port_encoding_x (port, NULL);
+
+  return port;
+}
+#undef FUNC_NAME
+
+SCM_INTERNAL SCM scm_i_mkstemp_x (SCM, SCM);
+SCM_DEFINE (scm_i_mkstemp_x, "mkstemp!", 1, 1, 0,
 	    (SCM tmpl, SCM mode),
 	    "Create a new unique file in the file system and return a new\n"
 	    "buffered port open for reading and writing to the file.\n"
@@ -1479,62 +1562,13 @@ SCM_DEFINE (scm_i_mkstemp, "mkstemp!", 1, 1, 0,
             "The optional @var{mode} argument specifies a mode, as a string\n"
             "in the same format that @code{open-file} takes.  It defaults\n"
             "to @code{\"w+\"}.")
-#define FUNC_NAME s_scm_i_mkstemp
+#define FUNC_NAME s_scm_i_mkstemp_x
 {
-  char *c_tmpl;
-  long mode_bits;
-  int rv;
-  int open_flags, is_binary;
-  SCM port;
-
-  SCM_VALIDATE_STRING (SCM_ARG1, tmpl);
-  if (!SCM_UNBNDP (mode))
-    SCM_VALIDATE_STRING (SCM_ARG2, mode);
-
-  /* Ensure tmpl is mutable.  */
-  scm_i_string_start_writing (tmpl);
-  scm_i_string_stop_writing ();
-
-  scm_dynwind_begin (0);
-
-  c_tmpl = scm_to_locale_string (tmpl);
-  scm_dynwind_free (c_tmpl);
-  if (SCM_UNBNDP (mode))
-    {
-      /* mkostemp will create a read/write file and add on additional
-         flags; open_flags just adjoins flags to that set.  */
-      open_flags = 0;
-      is_binary = 0;
-      mode_bits = SCM_OPN | SCM_RDNG | SCM_WRTNG;
-    }
-  else
-    {
-      open_flags = scm_i_mode_to_open_flags (mode, &is_binary, FUNC_NAME);
-      /* mkostemp(2) only defines O_APPEND, O_SYNC, and O_CLOEXEC to be
-         useful, as O_RDWR|O_CREAT|O_EXCL are implicitly added.  It also
-         notes that other flags may error on some systems, which turns
-         out to be the case.  Of those flags, O_APPEND is the only one
-         of interest anyway, so limit to that flag.  */
-      open_flags &= O_APPEND;
-      mode_bits = scm_i_mode_bits (mode);
-    }
-
-  SCM_SYSCALL (rv = mkostemp (c_tmpl, open_flags));
-  if (rv == -1)
-    SCM_SYSERROR;
-
-  scm_substring_move_x (scm_from_locale_string (c_tmpl),
+  SCM ret = scm_i_mkstemp (tmpl, mode);
+  scm_substring_move_x (scm_port_filename (ret),
 			SCM_INUM0, scm_string_length (tmpl),
 			tmpl, SCM_INUM0);
-
-  scm_dynwind_end ();
-
-  port = scm_i_fdes_to_port (rv, mode_bits, tmpl, 0);
-  if (is_binary)
-    /* Use the binary-friendly ISO-8859-1 encoding. */
-    scm_i_set_port_encoding_x (port, NULL);
-
-  return port;
+  return ret;
 }
 #undef FUNC_NAME
 
@@ -1544,6 +1578,40 @@ scm_mkstemp (SCM tmpl)
   return scm_i_mkstemp (tmpl, SCM_UNDEFINED);
 }
 
+#if HAVE_MKDTEMP
+SCM_DEFINE (scm_mkdtemp, "mkdtemp", 1, 0, 0,
+	    (SCM tmpl),
+            "Create a new unique directory in the file system named in\n"
+            "accordance with @var{tmpl}. The last six characters of the\n"
+            "template must be 'XXXXXX'.\n"
+            "\n"
+            "Upon success, this procedure returns the name of the\n"
+            "directory created.\n"
+            "\n"
+            "An error may be thrown if the template is incorrect or if\n"
+            "the directory could not be created.\n")
+#define FUNC_NAME s_scm_mkdtemp
+{
+  char *c_tmpl;
+  char *rv;
+  SCM new_dirname;
+
+  SCM_VALIDATE_STRING (SCM_ARG1, tmpl);
+
+  scm_dynwind_begin (0);
+  c_tmpl = scm_to_locale_string (tmpl);
+  scm_dynwind_free (c_tmpl);
+  SCM_SYSCALL (rv = mkdtemp (c_tmpl));
+  if (rv == NULL)
+    SCM_SYSERROR;
+  new_dirname = scm_from_locale_string (c_tmpl);
+  scm_dynwind_end ();
+
+  return new_dirname;
+}
+#undef FUNC_NAME
+#endif /* HAVE_MKDTEMP */
+
 
 /* Filename manipulation */
 
@@ -1608,11 +1676,20 @@ SCM_DEFINE (scm_basename, "basename", 1, 1, 0,
   c_filename = scm_to_utf8_string (filename);
   scm_dynwind_free (c_filename);
 
-  c_last_component = last_component (c_filename);
-  if (!c_last_component)
-    res = filename;
+  if (strcmp (c_filename, "/") == 0
+      || strcmp (c_filename, "//") == 0)
+    /* As per
+       <http://pubs.opengroup.org/onlinepubs/9699919799/functions/basename.html>,
+       "/" and "//" are treated specially.  */
+    res = scm_from_utf8_string ("/");
   else
-    res = scm_from_utf8_string (c_last_component);
+    {
+      c_last_component = last_component (c_filename);
+      if (!c_last_component)
+        res = filename;
+      else
+        res = scm_from_utf8_string (c_last_component);
+    }
   scm_dynwind_end ();
 
   if (!SCM_UNBNDP (suffix) &&
@@ -1753,7 +1830,7 @@ SCM_DEFINE (scm_opendir, "opendir", 1, 0, 0,
     SCM_SYSERROR;
 
   SCM_RETURN_NEWSMOB2 (scm_tc16_dir | (SCM_DIR_FLAG_OPEN << 16),
-		       ds, SCM_PACK_POINTER (mutex));
+		       ds, SCM_UNPACK (SCM_PACK_POINTER (mutex)));
 }
 #undef FUNC_NAME
 
@@ -1915,9 +1992,45 @@ scm_init_filesys ()
 #ifdef O_LARGEFILE  
   scm_c_define ("O_LARGEFILE", scm_from_int (O_LARGEFILE));
 #endif
+#ifdef O_IGNORE_CTTY
+  scm_c_define ("O_IGNORE_CTTY", scm_from_int (O_IGNORE_CTTY));
+#endif
+#ifdef O_NOLINK
+  scm_c_define ("O_NOLINK", scm_from_int (O_NOLINK));
+#endif
 #ifdef O_NOTRANS
   scm_c_define ("O_NOTRANS", scm_from_int (O_NOTRANS));
 #endif
+#ifdef O_NOFOLLOW
+  scm_c_define ("O_NOFOLLOW", scm_from_int (O_NOFOLLOW));
+#endif
+#ifdef O_PATH
+  scm_c_define ("O_PATH", scm_from_int (O_PATH));
+#endif
+#ifdef O_TMPFILE
+  scm_c_define ("O_TMPFILE", scm_from_int (O_TMPFILE));
+#endif
+#ifdef O_SHLOCK
+  scm_c_define ("O_SHLOCK", scm_from_int (O_SHLOCK));
+#endif
+#ifdef O_EXLOCK
+  scm_c_define ("O_EXLOCK", scm_from_int (O_EXLOCK));
+#endif
+#ifdef O_ASYNC
+  scm_c_define ("O_ASYNC", scm_from_int (O_ASYNC));
+#endif
+#ifdef O_NOATIME
+  scm_c_define ("O_NOATIME", scm_from_int (O_NOATIME));
+#endif
+#ifdef O_DIRECTORY
+  scm_c_define ("O_DIRECTORY", scm_from_int (O_DIRECTORY));
+#endif
+#ifdef O_CLOEXEC
+  scm_c_define ("O_CLOEXEC", scm_from_int (O_CLOEXEC));
+#endif
+#ifdef O_DIRECT
+  scm_c_define ("O_DIRECT", scm_from_int (O_DIRECT));
+#endif
 
 #ifdef F_DUPFD  
   scm_c_define ("F_DUPFD", scm_from_int (F_DUPFD));
@@ -1951,7 +2064,7 @@ scm_init_filesys ()
   scm_c_define ("X_OK", scm_from_int (X_OK));
   scm_c_define ("F_OK", scm_from_int (F_OK));
 
-  scm_dot_string = scm_from_locale_string (".");
+  scm_dot_string = scm_from_utf8_string (".");
 
 #include "filesys.x"
 }
