@@ -123,8 +123,6 @@ static void reset_abi_arg_iterator(struct abi_arg_iterator *iter, size_t argc,
 static void next_abi_arg(struct abi_arg_iterator *iter,
                          jit_operand_t *arg);
 
-static jit_gpr_t get_callr_temp (jit_state_t * _jit);
-
 jit_bool_t
 init_jit(void)
 {
@@ -270,22 +268,6 @@ get_temp_gpr(jit_state_t *_jit)
 #ifdef JIT_TMP1
     case 1:
       return JIT_TMP1;
-#endif
-#ifdef JIT_TMP2
-    case 2:
-      return JIT_TMP2;
-#endif
-#ifdef JIT_TMP3
-    case 3:
-      return JIT_TMP3;
-#endif
-#ifdef JIT_TMP4
-    case 4:
-      return JIT_TMP4;
-#endif
-#ifdef JIT_TMP5
-    case 5:
-      return JIT_TMP5;
 #endif
     default:
       abort();
@@ -576,8 +558,6 @@ jit_emit_addr(jit_state_t *j)
 # include "aarch64.c"
 #elif defined(__s390__) || defined(__s390x__)
 # include "s390.c"
-#elif defined(__riscv__) || defined(__riscv)
-# include "riscv.c"
 #endif
 
 #define JIT_IMPL_0(stem, ret) \
@@ -806,14 +786,6 @@ abi_mem_to_gpr(jit_state_t *_jit, enum jit_operand_abi abi,
   case JIT_OPERAND_ABI_INT16:
     jit_ldxi_s(_jit, dst, base, offset);
     break;
-  case JIT_OPERAND_ABI_FLOAT:
-  {
-    jit_fpr_t tmp = get_temp_fpr(_jit);
-    jit_ldxi_f(_jit, tmp, base, offset);
-    jit_movr_i_f(_jit, dst, tmp);
-    unget_temp_fpr(_jit);
-    break;
-  }
 #if __WORDSIZE == 32
   case JIT_OPERAND_ABI_UINT32:
   case JIT_OPERAND_ABI_POINTER:
@@ -830,14 +802,6 @@ abi_mem_to_gpr(jit_state_t *_jit, enum jit_operand_abi abi,
   case JIT_OPERAND_ABI_INT64:
     jit_ldxi_l(_jit, dst, base, offset);
     break;
-  case JIT_OPERAND_ABI_DOUBLE:
-  {
-    jit_fpr_t tmp = get_temp_fpr(_jit);
-    jit_ldxi_d(_jit, tmp, base, offset);
-    jit_movr_l_d(_jit, dst, tmp);
-    unget_temp_fpr(_jit);
-    break;
-  }
 #endif
   default:
     abort();
@@ -902,8 +866,7 @@ enum move_kind {
   MOVE_KIND_ENUM(IMM, MEM),
   MOVE_KIND_ENUM(GPR, MEM),
   MOVE_KIND_ENUM(FPR, MEM),
-  MOVE_KIND_ENUM(MEM, MEM),
-  MOVE_KIND_ENUM(FPR, GPR)
+  MOVE_KIND_ENUM(MEM, MEM)
 };
 #undef MOVE_KIND_ENUM
 
@@ -917,14 +880,6 @@ move_operand(jit_state_t *_jit, jit_operand_t dst, jit_operand_t src)
   case MOVE_GPR_TO_GPR:
     return jit_movr(_jit, dst.loc.gpr.gpr, src.loc.gpr.gpr);
 
-  case MOVE_FPR_TO_GPR:
-#if __WORDSIZE > 32
-    if (src.abi == JIT_OPERAND_ABI_DOUBLE)
-      return jit_movr_l_d(_jit, dst.loc.gpr.gpr, src.loc.fpr);
-    else
-#endif
-      return jit_movr_i_f(_jit, dst.loc.gpr.gpr, src.loc.fpr);
-
   case MOVE_MEM_TO_GPR:
     return abi_mem_to_gpr(_jit, src.abi, dst.loc.gpr.gpr, src.loc.mem.base,
                           src.loc.mem.offset);
@@ -1140,15 +1095,6 @@ jit_move_operands(jit_state_t *_jit, jit_operand_t *dst, jit_operand_t *src,
   enum move_status status[argc];
   for (size_t i = 0; i < argc; i++)
     status[i] = TO_MOVE;
-
-  // Mem-to-mem moves require a temp register but don't overwrite
-  // other argument registers.  Perform them first to free up the tmp
-  // for other uses.
-  for (size_t i = 0; i < argc; i++)
-    if ((status[i] == TO_MOVE)
-	&& (MOVE_KIND (src[i].kind, dst[i].kind) == MOVE_MEM_TO_MEM))
-      move_one(_jit, dst, src, argc, status, i);
-
   for (size_t i = 0; i < argc; i++)
     if (status[i] == TO_MOVE)
       move_one(_jit, dst, src, argc, status, i);
@@ -1209,9 +1155,6 @@ static const jit_gpr_t user_callee_save_gprs[] = {
 #endif
 #ifdef JIT_V9
   , JIT_V9
-#endif
-#ifdef JIT_V10
-  , JIT_V10
 #endif
  };
 
@@ -1240,18 +1183,6 @@ static const jit_fpr_t user_callee_save_fprs[] = {
 #ifdef JIT_VF7
   , JIT_VF7
 #endif
-#ifdef JIT_VF8
-  , JIT_VF8
-#endif
-#ifdef JIT_VF9
-  , JIT_VF9
-#endif
-#ifdef JIT_VF10
-  , JIT_VF10
-#endif
-#ifdef JIT_VF11
-  , JIT_VF11
-#endif
 };
 
 #define ARRAY_SIZE(X) (sizeof (X)/sizeof ((X)[0]))
@@ -1304,23 +1235,11 @@ jit_leave_jit_abi(jit_state_t *_jit, size_t v, size_t vf, size_t frame_size)
 
 // Precondition: stack is already aligned.
 static size_t
-prepare_call_args(jit_state_t *_jit, size_t argc, jit_operand_t args[],
-		  jit_gpr_t *fun)
+prepare_call_args(jit_state_t *_jit, size_t argc, jit_operand_t args[])
 {
-  size_t count = argc + (fun == NULL ? 0 : 1);
-  jit_operand_t src[count];
-  jit_operand_t dst[count];
-
-  memcpy (src, args, sizeof (jit_operand_t) * argc);
-  if (fun != NULL) {
-    jit_gpr_t fun_tmp = argc == 0 ? *fun : get_callr_temp (_jit);
-    src[argc] = jit_operand_gpr (JIT_OPERAND_ABI_POINTER, *fun);
-    dst[argc] = jit_operand_gpr (JIT_OPERAND_ABI_POINTER, fun_tmp);
-    *fun = fun_tmp;
-  }
-
+  jit_operand_t dst[argc];
   struct abi_arg_iterator iter;
-
+  
   // Compute shuffle destinations and space for spilled arguments.
   reset_abi_arg_iterator(&iter, argc, args);
   for (size_t i = 0; i < argc; i++)
@@ -1345,7 +1264,7 @@ prepare_call_args(jit_state_t *_jit, size_t argc, jit_operand_t args[],
     }
   }
 
-  jit_move_operands(_jit, dst, src, count);
+  jit_move_operands(_jit, dst, args, argc);
 
   return stack_size;
 }
@@ -1353,7 +1272,7 @@ prepare_call_args(jit_state_t *_jit, size_t argc, jit_operand_t args[],
 void
 jit_calli(jit_state_t *_jit, jit_pointer_t f, size_t argc, jit_operand_t args[])
 {
-  size_t stack_bytes = prepare_call_args(_jit, argc, args, NULL);
+  size_t stack_bytes = prepare_call_args(_jit, argc, args);
 
   calli(_jit, (jit_word_t)f);
 
@@ -1363,7 +1282,7 @@ jit_calli(jit_state_t *_jit, jit_pointer_t f, size_t argc, jit_operand_t args[])
 void
 jit_callr(jit_state_t *_jit, jit_gpr_t f, size_t argc, jit_operand_t args[])
 {
-  size_t stack_bytes = prepare_call_args(_jit, argc, args, &f);
+  size_t stack_bytes = prepare_call_args(_jit, argc, args);
 
   callr(_jit, jit_gpr_regno(f));
 
@@ -1423,7 +1342,6 @@ alloc_literal_pool(jit_state_t *_jit, size_t capacity)
                  sizeof (struct jit_literal_pool_entry) * capacity);
   ASSERT (ret);
   ret->capacity = capacity;
-  ret->size = 0;
   reset_literal_pool(_jit, ret);
   return ret;
 }
