@@ -1,5 +1,5 @@
 ;;; Type analysis on CPS
-;;; Copyright (C) 2014-2019 Free Software Foundation, Inc.
+;;; Copyright (C) 2014-2021 Free Software Foundation, Inc.
 ;;;
 ;;; This library is free software: you can redistribute it and/or modify
 ;;; it under the terms of the GNU Lesser General Public License as
@@ -127,6 +127,7 @@
             type<=?
 
             ;; Interface for type inference.
+            constant-type
             infer-types
             lookup-pre-type
             lookup-post-type
@@ -342,8 +343,8 @@
 minimum, and maximum."
   (define (return type val)
     (if val
-        (make-type-entry type val val)
-        (make-type-entry type -inf.0 +inf.0)))
+        (values type val val)
+        (values type -inf.0 +inf.0)))
   (cond
    ((number? val)
     (cond
@@ -356,8 +357,8 @@ minimum, and maximum."
               val))
      ((eqv? (imag-part val) 0)
       (if (nan? val)
-          (make-type-entry &flonum -inf.0 +inf.0)
-          (make-type-entry
+          (values &flonum -inf.0 +inf.0)
+          (values
            (if (exact? val) &fraction &flonum)
            (if (rational? val) (inexact->exact (floor val)) val)
            (if (rational? val) (inexact->exact (ceiling val)) val))))
@@ -367,6 +368,7 @@ minimum, and maximum."
    ((eq? val #t) (return &special-immediate &true))
    ((eq? val #f) (return &special-immediate &false))
    ((eqv? val *unspecified*) (return &special-immediate &unspecified))
+   ((eof-object? val) (return &special-immediate &eof))
    ((char? val) (return &char (char->integer val)))
    ((symbol? val) (return &symbol #f))
    ((keyword? val) (return &keyword #f))
@@ -382,6 +384,13 @@ minimum, and maximum."
 
    (else (error "unhandled constant" val))))
 
+(define (constant-type-entry val)
+  "Compute the type and range of VAL.  Return three values: the type,
+minimum, and maximum."
+  (call-with-values (lambda () (constant-type val))
+    (lambda (type min max)
+      (make-type-entry type min max))))
+
 (define *type-checkers* (make-hash-table))
 (define *type-inferrers* (make-hash-table))
 
@@ -570,25 +579,28 @@ minimum, and maximum."
 ;;; Generic effect-free predicates.
 ;;;
 
-(define-syntax-rule (define-special-immediate-predicate-inferrer pred imm)
-  (define-predicate-inferrer (pred val true?)
+(define-syntax-rule (infer-constant-comparison ctype cval val true?)
+  (let ()
     (define (range-subtract lo hi x)
       (values (if (eqv? lo x) (1+ lo) lo)
               (if (eqv? hi x) (1- hi) hi)))
-    (cond
-     (true? (restrict! val &special-immediate imm imm))
-     (else
-      (when (eqv? (&type val) &special-immediate)
-        (let-values (((lo hi) (range-subtract (&min val) (&max val) imm)))
-          (restrict! val &special-immediate lo hi)))))))
+   (cond
+    (true? (restrict! val ctype cval cval))
+    (else
+     (when (eqv? (&type val) ctype)
+       (let-values (((lo hi) (range-subtract (&min val) (&max val) cval)))
+         (restrict! val ctype lo hi)))))))
 
-(define-special-immediate-predicate-inferrer eq-nil? &nil)
-(define-special-immediate-predicate-inferrer eq-eol? &null)
-(define-special-immediate-predicate-inferrer eq-false? &false)
-(define-special-immediate-predicate-inferrer eq-true? &true)
-(define-special-immediate-predicate-inferrer unspecified? &unspecified)
-(define-special-immediate-predicate-inferrer undefined? &undefined)
-(define-special-immediate-predicate-inferrer eof-object? &eof)
+(define-predicate-inferrer/param (eq-constant? c val true?)
+  (call-with-values (lambda () (constant-type c))
+    (lambda (ctype cval cval*)
+      ;; Either (= cval cval*), or the value is meaningless for this type.
+      (infer-constant-comparison ctype cval val true?))))
+
+;; Can't usefully pass undefined as a parameter to eq-constant?, so we
+;; keep its special predicate.
+(define-predicate-inferrer (undefined? val true?)
+  (infer-constant-comparison &special-immediate &undefined val true?))
 
 ;; Various inferrers rely on these having contiguous values starting from 0.
 (eval-when (expand)
@@ -702,7 +714,7 @@ minimum, and maximum."
 
 
 (define-type-inferrer/param (load-const/unlikely param result)
-  (let ((ent (constant-type param)))
+  (let ((ent (constant-type-entry param)))
     (define! result (type-entry-type ent)
       (type-entry-min ent) (type-entry-max ent))))
 
@@ -728,19 +740,27 @@ minimum, and maximum."
     ('vector &vector)
     ('string &string)
     ('stringbuf &string)
+    ('symbol &symbol)
     ('bytevector &bytevector)
     ('box &box)
     ('closure &procedure)
     ('struct &struct)
     ('atomic-box &all-types)))
 
+(define (annotation->mutable-type ann)
+  (match ann
+    ('vector &mutable-vector)
+    (_ (annotation->type ann))))
+
 (define-type-inferrer/param (allocate-words param size result)
-  (define! result (annotation->type param) (&min/0 size) (&max/scm-size size)))
+  (define! result (annotation->mutable-type param)
+    (&min/0 size) (&max/scm-size size)))
 
 (define-type-inferrer/param (allocate-words/immediate param result)
   (match param
     ((annotation . size)
-     (define! result (annotation->type annotation) size size))))
+     (define! result (annotation->mutable-type annotation)
+       size size))))
 
 (define-type-inferrer-aliases allocate-words allocate-pointerless-words)
 (define-type-inferrer-aliases allocate-words/immediate
@@ -761,15 +781,15 @@ minimum, and maximum."
   (restrict! obj (annotation->type param) -inf.0 +inf.0)
   (define! result &all-types -inf.0 +inf.0))
 (define-type-inferrer/param (scm-set!/tag param obj val)
-  (restrict! obj (annotation->type param) -inf.0 +inf.0))
+  (restrict! obj (annotation->mutable-type param) -inf.0 +inf.0))
 
 (define-type-inferrer/param (scm-set! param obj idx val)
-  (restrict! obj (annotation->type param) (1+ (&min/0 idx)) +inf.0))
+  (restrict! obj (annotation->mutable-type param) (1+ (&min/0 idx)) +inf.0))
 
 (define-type-inferrer/param (scm-set!/immediate param obj val)
   (match param
     ((annotation . idx)
-     (restrict! obj (annotation->type annotation) (1+ idx) +inf.0))))
+     (restrict! obj (annotation->mutable-type annotation) (1+ idx) +inf.0))))
 
 (define-type-inferrer/param (word-ref param obj idx result)
   (restrict! obj (annotation->type param)
@@ -783,12 +803,12 @@ minimum, and maximum."
      (define! result &u64 0 &u64-max))))
 
 (define-type-inferrer/param (word-set! param obj idx word)
-  (restrict! obj (annotation->type param) (1+ (&min/0 idx)) +inf.0))
+  (restrict! obj (annotation->mutable-type param) (1+ (&min/0 idx)) +inf.0))
 
 (define-type-inferrer/param (word-set!/immediate param obj word)
   (match param
     ((annotation . idx)
-     (restrict! obj (annotation->type annotation) (1+ idx) +inf.0))))
+     (restrict! obj (annotation->mutable-type annotation) (1+ idx) +inf.0))))
 
 (define-type-inferrer/param (pointer-ref/immediate param obj result)
   (define! result &other-heap-object -inf.0 +inf.0))
@@ -1092,7 +1112,7 @@ minimum, and maximum."
                          (+ (&min a) (&min b))
                          (+ (&max a) (&max b))))
 (define-type-inferrer/param (add/immediate param a result)
-  (let ((b-type (type-entry-type (constant-type param))))
+  (let ((b-type (type-entry-type (constant-type-entry param))))
     (define-binary-result! (&type a) b-type result #t
       (+ (&min a) param)
       (+ (&max a) param))))
@@ -1136,7 +1156,7 @@ minimum, and maximum."
                          (- (&min a) (&max b))
                          (- (&max a) (&min b))))
 (define-type-inferrer/param (sub/immediate param a result)
-  (let ((b-type (type-entry-type (constant-type param))))
+  (let ((b-type (type-entry-type (constant-type-entry param))))
     (define-binary-result! (&type a) b-type result #t
       (- (&min a) param)
       (- (&max a) param))))
@@ -1241,9 +1261,10 @@ minimum, and maximum."
        (not (<= (&min b) 0 (&max b)))))
 (define-type-checker (fdiv a b) #t)
 (define (div-result-range min-a max-a min-b max-b)
-  (if (<= min-b 0 max-b)
-      ;; If the range of the divisor crosses 0, the result spans
-      ;; the whole range.
+  (if (or (<= min-b 0 max-b)
+          (< max-b min-b))
+      ;; If the range of the divisor crosses 0, or if we are in
+      ;; unreachable code, the result spans the whole range.
       (values -inf.0 +inf.0)
       ;; Otherwise min-b and max-b have the same sign, and cannot both
       ;; be infinity.
@@ -1274,44 +1295,45 @@ minimum, and maximum."
       (lambda (min max)
         (define! result &f64 min max)))))
 
+(define &integer (logior &exact-integer &flonum))
+
 (define-type-checker (quo a b)
-  (and (check-type a &exact-integer -inf.0 +inf.0)
-       (check-type b &exact-integer -inf.0 +inf.0)
+  (and (check-type a &integer -inf.0 +inf.0)
+       (check-type b &integer -inf.0 +inf.0)
        ;; We only know that there will not be an exception if b is not
        ;; zero.
        (not (<= (&min b) 0 (&max b)))))
 (define-type-inferrer (quo a b result)
-  (restrict! a &exact-integer -inf.0 +inf.0)
-  (restrict! b &exact-integer -inf.0 +inf.0)
-  (define! result &exact-integer -inf.0 +inf.0))
+  (restrict! a &integer -inf.0 +inf.0)
+  (restrict! b &integer -inf.0 +inf.0)
+  (define! result (logand (logior (&type a) (&type b)) &integer)
+    -inf.0 +inf.0))
 
 (define-type-checker-aliases quo rem)
 (define-type-inferrer (rem a b result)
-  (restrict! a &exact-integer -inf.0 +inf.0)
-  (restrict! b &exact-integer -inf.0 +inf.0)
+  (restrict! a &integer -inf.0 +inf.0)
+  (restrict! b &integer -inf.0 +inf.0)
   ;; Same sign as A.
-  (let ((max-abs-rem (1- (max (abs (&min b)) (abs (&max b))))))
-    (cond
-     ((< (&min a) 0)
-      (define-exact-integer! result
-        (- max-abs-rem)
-        (if (< 0 (&max a)) max-abs-rem 0)))
-     (else
-      (define-exact-integer! result 0 max-abs-rem)))))
+  (let* ((max-abs-rem (1- (max (abs (&min b)) (abs (&max b)))))
+         (t (logand (logior (&type a) (&type b)) &integer))
+         (min-rem (if (< (&min a) 0) (- max-abs-rem) 0))
+         (max-rem (if (< 0 (&max a)) max-abs-rem 0)))
+    (if (type<=? t &exact-integer)
+        (define-exact-integer! result min-rem max-rem)
+        (define! result t min-rem max-rem))))
 
 (define-type-checker-aliases quo mod)
 (define-type-inferrer (mod a b result)
-  (restrict! a &exact-integer -inf.0 +inf.0)
-  (restrict! b &exact-integer -inf.0 +inf.0)
+  (restrict! a &integer -inf.0 +inf.0)
+  (restrict! b &integer -inf.0 +inf.0)
   ;; Same sign as B.
-  (let ((max-abs-mod (1- (max (abs (&min b)) (abs (&max b))))))
-    (cond
-     ((< (&min b) 0)
-      (define-exact-integer! result
-        (- max-abs-mod)
-        (if (< 0 (&max b)) max-abs-mod 0)))
-     (else
-      (define-exact-integer! result 0 max-abs-mod)))))
+  (let* ((max-abs-mod (1- (max (abs (&min b)) (abs (&max b)))))
+         (t (logand (logior (&type a) (&type b)) &integer))
+         (min-mod (if (< (&min b) 0) (- max-abs-mod) 0))
+         (max-mod (if (< 0 (&max b)) max-abs-mod 0)))
+    (if (type<=? t &exact-integer)
+        (define-exact-integer! result min-mod max-mod)
+        (define! result t min-mod max-mod))))
 
 ;; Predicates.
 (define-syntax-rule (define-type-predicate-result val result type)
@@ -1345,7 +1367,7 @@ minimum, and maximum."
 
 (define-type-aliases inf? nan?)
 
-(define-simple-type (even? &exact-integer)
+(define-simple-type (even? &integer)
   (&special-immediate &false &true))
 (define-type-aliases even? odd?)
 
@@ -1358,7 +1380,7 @@ minimum, and maximum."
     ;; shifts that might exceed that range.
     (cond
      ((inf? val) val) ; Preserves sign.
-     ((< count 64) (ash val (max count 0)))
+     ((< count 64) (ash val count))
      ((zero? val) 0)
      ((positive? val) +inf.0)
      (else -inf.0)))
@@ -1374,6 +1396,7 @@ minimum, and maximum."
                                              (&min/0 count)
                                              (&max/u64 count))))
     (define-exact-integer! result min max)))
+(define-simple-type-checker (lsh/immediate &exact-integer))
 (define-type-inferrer/param (lsh/immediate count val result)
   (restrict! val &exact-integer -inf.0 +inf.0)
   (let-values (((min max) (compute-ash-range (&min val)
@@ -1387,6 +1410,7 @@ minimum, and maximum."
                                              (- (&min/0 count))
                                              (- (&max/u64 count)))))
     (define-exact-integer! result min max)))
+(define-simple-type-checker (rsh/immediate &exact-integer))
 (define-type-inferrer/param (rsh/immediate count val result)
   (restrict! val &exact-integer -inf.0 +inf.0)
   (let-values (((min max) (compute-ash-range (&min val)
@@ -1848,6 +1872,7 @@ where (A0 <= A <= A1) and (B0 <= B <= B1)."
     (($ $kargs _ _ ($ $throw)) 0)
     (($ $kargs _ _ ($ $continue)) 1)
     (($ $kargs _ _ (or ($ $branch) ($ $prompt))) 2)
+    (($ $kargs _ _ ($ $switch kf kt*)) (1+ (length kt*)))
     (($ $kfun src meta self tail clause) (if clause 1 0))
     (($ $kclause arity body alt) (if alt 2 1))
     (($ $kreceive) 1)
@@ -2016,7 +2041,7 @@ maximum, where type is a bitset as a fixnum."
          (($ $kargs (_) (var))
           (let ((entry (match exp
                          (($ $const val)
-                          (constant-type val))
+                          (constant-type-entry val))
                          ((or ($ $prim) ($ $fun) ($ $const-fun) ($ $code))
                           ;; Could be more precise here.
                           (make-type-entry &procedure -inf.0 +inf.0)))))
@@ -2044,6 +2069,24 @@ maximum, where type is a bitset as a fixnum."
          ;; The "normal" continuation is the #f branch.
          (propagate2 kf (infer-primcall types 0 op param args #f)
                      kt (infer-primcall types 1 op param args #f)))
+        (($ $kargs names vars ($ $switch kf kt* src arg))
+         (define (restrict-index min max)
+           (restrict-var types arg (make-type-entry &u64 min max)))
+         (define (visit-default typev)
+           (let ((types (restrict-index (length kt*) &u64-max)))
+             (propagate-types label typev 0 kf types)))
+         (define (visit-target typev k i)
+           (let ((types (restrict-index i i)))
+             (propagate-types label typev (1+ i) k types)))
+         (call-with-values (lambda () (visit-default typev))
+           (lambda (changed typev)
+             (let lp ((kt* kt*) (i 0) (changed changed) (typev typev))
+               (match kt*
+                 (() (values changed typev))
+                 ((kt . kt*)
+                  (call-with-values (lambda () (visit-target typev kt i))
+                    (lambda (changed* typev)
+                      (lp kt* (1+ i) (append changed* changed) typev)))))))))
         (($ $kargs names vars ($ $prompt k kh src escape? tag))
          ;; The "normal" continuation enters the prompt.
          (propagate2 k types kh types))
@@ -2055,9 +2098,14 @@ maximum, where type is a bitset as a fixnum."
             (propagate1 k (adjoin-vars types vars all-types-entry)))))
         (($ $kfun src meta self tail clause)
          (if clause
-             (propagate1 clause (if self
-                                    (adjoin-var types self all-types-entry)
-                                    types))
+             (let ((types (if self
+                              (adjoin-var types self all-types-entry)
+                              types)))
+               (propagate1 clause
+                           (match (intmap-ref conts clause)
+                             (($ $kargs _ defs)
+                              (adjoin-vars types defs all-types-entry))
+                             (_ types))))
              (propagate0)))
         (($ $kclause arity kbody kalt)
          (match (intmap-ref conts kbody)
